==> inner-apps-0.03.tar.gz: inner-apps-0.03/ipfwdump.c <==
/*
%%% copyright-cmetz
This software is Copyright 1996 by Craig Metz, All Rights Reserved.
The Inner Net License Version 2 applies to this software.
You should have received a copy of the license with this software. If
you didn't get a copy, you may request one from <license@inner.net>.

v0.02 - NOT FOR REDISTRIBUTION
*/
#define NAME "ipfwdump"
#define VERSION "v0.02 - NOT FOR REDISTRIBUTION"
#define USAGE "[-Vh] [<savefile>] [<device>]"

#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <fcntl.h>
#include <getopt.h>

#include "inner-apps.h"

#define BUFFER_SZ 65536
void *buffer = NULL;
char *savefile = "/usr/adm/ipfw.tcpdump";
char *devicefile = "/dev/netlink-firewall";

int debug = 0;

#define PCAP_MAGIC 0xa1b2c3d4
#define PCAP_VERSION_MAJOR 2

struct pcap_file_header {
  u_int32_t magic;
  u_int16_t version_major;
  u_int16_t version_minor;
    int32_t thiszone;
  u_int32_t sigfigs;
  u_int32_t snaplen;
  u_int32_t linktype;
};

struct pcap_null_pkthdr {
  struct timeval ts;
  u_int32_t caplen;
  u_int32_t len;
  u_int32_t af;
};

int main(int argc, char **argv)
{
  int i, in, out;
  struct pcap_null_pkthdr pkthdr;

  INIT(argc, argv);

  while ((i = getopt(argc, argv, STDOPTS_FLAGS "d" )) != EOF) {
    switch(i) {
      case 'd':
        debug = 1;
        break;
      STDOPTS_CASES
    };
  };

  if (optind < argc)
    savefile = argv[optind++];

  if (optind < argc)
    devicefile = argv[optind++];

  {
  struct pcap_file_header h;

  if ((out = open(savefile, O_RDONLY)) < 0) {
    if (errno != ENOENT) {
      fprintf(stderr, "%s: can't open %s: %s(%d)\n", myname, savefile, strerror(errno), errno);
      exit(1);
    };
    errno = 0;
    if ((out = open(savefile, O_WRONLY | O_APPEND | O_CREAT, 0600)) < 0) {
      fprintf(stderr, "%s: can't open %s: %s(%d)\n", myname, savefile, strerror(errno), errno);
      exit(1);
    };
    memset(&h, 0, sizeof(struct pcap_file_header));
    h.magic = PCAP_MAGIC;
    h.version_major = PCAP_VERSION_MAJOR;
    {
    struct timezone tz;
    if (gettimeofday(NULL, &tz) < 0) {
      fprintf(stderr, "%s: gettimeofday(NULL, &tz): %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
    h.thiszone = tz.tz_minuteswest / 60;
    };
    h.snaplen = BUFFER_SZ;
    if (write(out, &h, sizeof(struct pcap_file_header)) != sizeof(struct pcap_file_header)) {
      fprintf(stderr, "%s: write: %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
  } else {
    if (read(out, &h, sizeof(struct pcap_file_header)) != sizeof(struct pcap_file_header)) {
      fprintf(stderr, "%s: can't read header from %s.\n", myname, savefile);
      exit(1);
    };    
    if ((h.magic != PCAP_MAGIC) || (h.version_major < PCAP_VERSION_MAJOR) || h.linktype) {
      fprintf(stderr, "%s: header from %s is invalid and/or incompatible with this program.\n", myname, savefile);
      exit(1);
    };
    close(out);
    if ((out = open(savefile, O_WRONLY | O_APPEND)) < 0) {
      fprintf(stderr, "%s: can't open %s for append: %s(%d)\n", myname, savefile, strerror(errno), errno);
      exit(1);
    };
  };
  };

  if ((in = open(devicefile, O_RDONLY)) < 0) {
    fprintf(stderr, "%s: can't open %s: %s(%d)\n", myname, devicefile, strerror(errno), errno);
    exit(1);
  };

  if (!(buffer = malloc(BUFFER_SZ))) {
    fprintf(stderr, "%s: can't allocate %d bytes of memory: %s(%d)\n", myname, BUFFER_SZ, strerror(errno), errno);
    exit(1);
  };

  memset(&pkthdr, 0, sizeof(struct pcap_null_pkthdr));
  pkthdr.af = htonl(AF_INET);

  while(1) {
    if (debug)
      memset(buffer, 0, BUFFER_SZ);
    if ((i = read(in, buffer, BUFFER_SZ)) < 0) {
      fprintf(stderr, "%s: read: %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
    if (debug)
      fprintf(stderr, "%s: got a packet of length %d: %02x%02x%02x%02x...\n", myname, i, ((u_int8_t *)buffer)[0], ((u_int8_t *)buffer)[1], ((u_int8_t *)buffer)[2], ((u_int8_t *)buffer)[3]);
    if (gettimeofday(&pkthdr.ts, NULL) < 0) {
      fprintf(stderr, "%s: gettimeofday(&pkthdr.ts, NULL): %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
    pkthdr.caplen = pkthdr.len = i + 4;
    if (write(out, &pkthdr, sizeof(struct pcap_null_pkthdr)) != sizeof(struct pcap_null_pkthdr)) {
      fprintf(stderr, "%s: write: %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
    if (write(out, buffer, i) != i) {
      fprintf(stderr, "%s: fwrite: %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
  };

  return 0;
};

==> inner-apps-0.05.tar.gz: inner-apps-0.05/ipfwdump.c <==
/*
%%% copyright-cmetz-97
This software is Copyright 1997 by Craig Metz, All Rights Reserved.
The Inner Net License Version 2 applies to this software.
You should have received a copy of the license with this software. If
you didn't get a copy, you may request one from <license@inner.net>.

*/
#define NAME "ipfwdump"
#define VERSION "v0.03"
#define USAGE "[<savefile>] [<device>]"

#include <sys/types.h>
#include <sys/time.h>
#include <stdio.h>
#include <fcntl.h>
#include <getopt.h>

#include "inner-apps.h"

#define BUFFER_SZ 65536
void *buffer = NULL;
char *savefile = "/usr/adm/ipfw.tcpdump";
char *devicefile = "/dev/netlink-firewall";

#define PCAP_MAGIC 0xa1b2c3d4
#define PCAP_VERSION_MAJOR 2

struct pcap_file_header {
  u_int32_t magic;
  u_int16_t version_major;
  u_int16_t version_minor;
    int32_t thiszone;
  u_int32_t sigfigs;
  u_int32_t snaplen;
  u_int32_t linktype;
};

struct pcap_null_pkthdr {
  struct timeval ts;
  u_int32_t caplen;
  u_int32_t len;
  u_int32_t af;
};

int main(int argc, char **argv)
{
  int i, in, out;
  struct pcap_null_pkthdr pkthdr;

  INIT(argc, argv);

  while ((i = getopt(argc, argv, STDOPTS_FLAGS)) != EOF) {
    switch(i) {
      STDOPTS_CASES
    };
  };

  if (optind < argc)
    savefile = argv[optind++];

  if (optind < argc)
    devicefile = argv[optind++];

  {
  struct pcap_file_header h;

  if ((out = open(savefile, O_RDONLY)) < 0) {
    if (errno != ENOENT) {
      fprintf(stderr, "%s: can't open %s: %s(%d)\n", myname, savefile, strerror(errno), errno);
      exit(1);
    };
    errno = 0;
    if ((out = open(savefile, O_WRONLY | O_APPEND | O_CREAT, 0600)) < 0) {
      fprintf(stderr, "%s: can't open %s: %s(%d)\n", myname, savefile, strerror(errno), errno);
      exit(1);
    };
    memset(&h, 0, sizeof(struct pcap_file_header));
    h.magic = PCAP_MAGIC;
    h.version_major = PCAP_VERSION_MAJOR;
    {
    struct timezone tz;
    if (gettimeofday(NULL, &tz) < 0) {
      fprintf(stderr, "%s: gettimeofday(NULL, &tz): %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
    h.thiszone = tz.tz_minuteswest / 60;
    };
    h.snaplen = BUFFER_SZ;
    if (write(out, &h, sizeof(struct pcap_file_header)) != sizeof(struct pcap_file_header)) {
      fprintf(stderr, "%s: write: %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
  } else {
    if (read(out, &h, sizeof(struct pcap_file_header)) != sizeof(struct pcap_file_header)) {
      fprintf(stderr, "%s: can't read header from %s.\n", myname, savefile);
      exit(1);
    };    
    if ((h.magic != PCAP_MAGIC) || (h.version_major < PCAP_VERSION_MAJOR) || h.linktype) {
      fprintf(stderr, "%s: header from %s is invalid and/or incompatible with this program.\n", myname, savefile);
      exit(1);
    };
    close(out);
    if ((out = open(savefile, O_WRONLY | O_APPEND)) < 0) {
      fprintf(stderr, "%s: can't open %s for append: %s(%d)\n", myname, savefile, strerror(errno), errno);
      exit(1);
    };
  };
  };

  if ((in = open(devicefile, O_RDONLY)) < 0) {
    fprintf(stderr, "%s: can't open %s: %s(%d)\n", myname, devicefile, strerror(errno), errno);
    exit(1);
  };

  if (!(buffer = malloc(BUFFER_SZ))) {
    fprintf(stderr, "%s: can't allocate %d bytes of memory: %s(%d)\n", myname, BUFFER_SZ, strerror(errno), errno);
    exit(1);
  };

  memset(&pkthdr, 0, sizeof(struct pcap_null_pkthdr));
  pkthdr.af = htonl(AF_INET);

  while(1) {
    if (debug)
      memset(buffer, 0, BUFFER_SZ);
    if ((i = read(in, buffer, BUFFER_SZ)) < 0) {
      fprintf(stderr, "%s: read: %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
    if (debug)
      fprintf(stderr, "%s: got a packet of length %d: %02x%02x%02x%02x...\n", myname, i, ((u_int8_t *)buffer)[0], ((u_int8_t *)buffer)[1], ((u_int8_t *)buffer)[2], ((u_int8_t *)buffer)[3]);
    if (gettimeofday(&pkthdr.ts, NULL) < 0) {
      fprintf(stderr, "%s: gettimeofday(&pkthdr.ts, NULL): %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
    pkthdr.caplen = pkthdr.len = i + 4;
    if (write(out, &pkthdr, sizeof(struct pcap_null_pkthdr)) != sizeof(struct pcap_null_pkthdr)) {
      fprintf(stderr, "%s: write: %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
    if (write(out, buffer, i) != i) {
      fprintf(stderr, "%s: fwrite: %s(%d)\n", myname, strerror(errno), errno);
      exit(1);
    };
  };

  return 0;
};
